"""
Created by adam on 1/29/19
"""
__author__ = 'adam'


def determine_credit(submissions):
    """Adds submissions of zero length to the no-credit list.
    Adds others to the credit list.
    Returns a dictionary with keys credit and nocredit, with the lists as values
    """
    credit = []
    nocredit = []

    for s in submissions:
        if 'body' in s.keys() and s['body'] is not None and len(s['body']) > 2:
            credit.append(s['student_id'])
        else:
            nocredit.append(s['student_id'])

    return {'credit': credit, 'nocredit': nocredit}


if __name__ == '__main__':
    pass


"""
Created by adam on 1/7/19
"""
__author__ = 'adam'

from ipywidgets import widgets
from IPython.display import display

from assets.Configuration import InteractiveConfiguration

# -------- General
def make_text_input( input_dict ):
    """Creates a text input field. The given dictionary should have keys 'label' and 'handler'"""
    text = widgets.Text( description=input_dict[ 'label' ] )
    display( text )
    text.observe( input_dict[ 'handler' ] )


# -------- Canvas specific
def make_canvas_token_input():
    """Creates the input field for the canvas api token"""
    canvas_token_input = {'label' : 'Canvas token', 'handler': InteractiveConfiguration.handle_token_entry}
    make_text_input(canvas_token_input)


def make_course_ids_input():
    """
    Make box and buttons for course id entry.
    On submit button click, class ids updates with the value in the text box
    """
    t = widgets.Text(description='Class id')
    submit = widgets.Button(description='Add id')
    reset = widgets.Button(description='Reset ids')
    out = widgets.Output(layout={'border': '1px solid black'})

    def handle_add(change):
        InteractiveConfiguration.add_course_id(t.value)
        # Clear the box
        t.value = ''
        show_class_ids()

    def handle_reset(change):
        InteractiveConfiguration.reset_course_ids()
        show_class_ids()

    def show_class_ids():
        out.clear_output()
        with out:
            print('    ----Canvas class ids----    ')
            for cid in InteractiveConfiguration.course_ids:
                print(cid)

    submit.on_click(handle_add)
    reset.on_click(handle_reset)
    button_box = widgets.HBox([reset, submit])
    input_box = widgets.VBox([t, button_box])
    display(widgets.HBox([input_box, out]))


def make_general_reset_button():
    rb = widgets.Button(description='Clear canvas token and class ids')

    def reset_all(change):
        InteractiveConfiguration.reset_config()

    display(rb.on_click(reset_all))

if __name__ == '__main__':
    pass


"""
Created by adam on 1/31/19
"""
__author__ = 'adam'
import configparser
import os, sys


class Configuration( object ):
    archive_folder = False
    course_ids = [ ]
    canvas_token = False
    canvas_url_base = False
    log_folder = False

    @classmethod
    def add_course_id( cls, course_id ):
        cls.course_ids.append( course_id )

    @classmethod
    def add_canvas_token( cls, token ):
        cls.canvas_token = token

    @classmethod
    def add_canvas_url_base( cls, url ):
        cls.canvas_url_base = url

    @classmethod
    def reset_course_ids( cls ):
        cls.course_ids = [ ]
        print("List of course ids is empty")

    @classmethod
    def reset_canvas_token( cls ):
        cls.canvas_token = False
        print("Canvas token reset to empty")

    @classmethod
    def reset_config( cls ):
        cls.reset_canvas_token()
        cls.reset_course_ids()


class InteractiveConfiguration( Configuration ):
    def __init__( self ):
        super().__init__()

    @classmethod
    def handle_token_entry( cls, event ):
        if event[ 'type' ] == 'change' and event[ 'name' ] == 'value':
            v = event[ 'new' ]
            cls.add_canvas_token( v )


class FileBasedConfiguration(Configuration):
    configuration = False
    file_path = False

    def __init__(self, configuration_file_path):
        super().__init__()
        FileBasedConfiguration.file_path = configuration_file_path

    @classmethod
    def read_config_file( cls):
        if not cls.file_path:
            # The file path could've been customized by instantiating the class
            # If that didn't happen, we go with the default
            # The folder containing the assets folder
            cls.proj_base = os.path.abspath( os.path.dirname( os.path.dirname( __file__ )) )
            # All login credentials are defined in files here.
            # THIS CONTENTS OF THIS FOLDER MUST NOT BE COMMITTED TO VERSION CONTROL!
            CREDENTIALS_FOLDER_PATH = "%s/private" % cls.proj_base
            cls.file_path = "%s/canvas-credentials.ini" % CREDENTIALS_FOLDER_PATH

        cls.configuration = configparser.ConfigParser()
        cls.configuration.read( cls.file_path )
        print( "Reading credentials and settings from %s" % cls.file_path  )

    @classmethod
    def load( cls ):
        cls.read_config_file()
        cls.load_token()
        cls.load_url_base()
        cls.load_local_filepaths()
        cls.load_section_ids()

    @classmethod
    def load_section_ids( cls ):
        try:
            for v in cls.configuration['sections'].values():
                cls.add_course_id(v)
        except:
            pass

    @classmethod
    def load_local_filepaths( cls ):
        root = os.getenv( "HOME" )
        cls.archive_folder = "%s/%s" % (root, cls.configuration[ 'folders' ].get( 'STUDENT_WORK_ARCHIVE_FOLDER' ))
        cls.log_folder = "%s/%s" % (root, cls.configuration[ 'folders' ].get( 'LOG_FOLDER' ))
        cls.data_folder = '%s/data' % cls.proj_base

    @classmethod
    def load_token( cls ):
        if not cls.configuration:
            cls.read_config_file()

        cls.add_canvas_token(cls.configuration[ 'credentials' ].get( 'TOKEN' ))

    @classmethod
    def load_url_base( cls ):
        cls.add_canvas_url_base(cls.configuration[ 'url' ].get( 'BASE' ))


if __name__ == '__main__':
    FileBasedConfiguration.load()
    print(FileBasedConfiguration.canvas_token)



"""
Tools for uploading scores to canvas for journals

Created by adam on 1/29/19
"""
__author__ = 'adam'

import requests

from assets.UrlTools import make_url
from assets.RequestTools import make_request_header
from assets.DataManagement import DataStore


def upload_students_receiving_credit( store: DataStore ):
    """Makes the requests to the server to assign each of the students credit
    for the assignment
    """
    data = {'submission' : {'posted_grade' : 'pass'}}

    url = make_url(store.course_id, 'assignments')
    url = "%s/%s/submissions" % (url, store.assignment_id)

    for s in store.credit:
        surl = "%s/%s" % (url, s)
        print('credit', surl)
        requests.put(surl, headers=make_request_header(), json=data)


def assign_no_credit(course_id, assignment_id, students):
    """Makes the requests to the server to assign each of the students no credit
    for the assignment
    """
    data = {'submission' : {'posted_grade' : 'fail'}}

    url = make_url(course_id, 'assignments')
    url = "%s/%s/submissions" % (url, assignment_id)

    for s in students:
        surl = "%s/%s" % (url, s)
        print('no credit', surl)
#         requests.put(surl, headers=make_request_header(), json=data)

if __name__ == '__main__':
    pass


"""
Created by adam on 10/26/18
"""
__author__ = 'adam'

from assets.JournalsFileTools import load_words_to_ignore

def filter_out_terms(combined):
    """Loads the strings to ignore from IGNORE_FILE and returns the filtered wordbag"""
    exclude = load_words_to_ignore()
    #     exclude = ['/p','style=', 'br', '“', 'ethics', '”', 'm','’','p', 'picture-uploaded', 'div', '/div',
    #               '305', 'september', '2018', 'text-align', 'class=', 'span', 'swenson', 'phil', 'philosophy',
    #                '-webkit-standard', 'font', 'face='
    #               ]
    exclude += [ "%s" % i for i in range(0,10)]
    #     print(exclude)

    for k in combined.keys():
        combined[k] = [w for w in combined[k] if w not in exclude]
    return combined


if __name__ == '__main__':
    pass


"""
Created by adam on 2/1/19
"""
from typing import List, Any

__author__ = 'adam'


class DataStore(object):
    """All downloaded and score data gets stored in this object
    Methods using the data will expect an instance.
    """
    credit: List[ int ]

    def __init__(self, assignment_id=None,  assignment_name=None, course_id=None):
        self.course_id = course_id
        self.assignment_name = assignment_name
        self.assignment_id = assignment_id
        # List of ids from students receiving credit
        self.credit = []
        self.no_credit = []
        self.submissions = []

    def print_counts( self ):
        m = "Tentatively assigning credit to {} submissions; no credit to {} submissions."
        print(m.format(len(self.credit), len(self.no_credit)))

    # @property
    # def ids_receiving_credit( self):
    #     # if assignment_id is not None:
    #     #     c = list(filter(lambda x: x['assignment'] == assignment_id, self.credit))[0]
    #     #     return c['ids']
    #     return self.credit[0].ids


if __name__ == '__main__':
    pass


"""
Created by adam on 9/21/18
"""
__author__ = 'adam'
import nltk
from nltk.corpus import stopwords
import string

remove = [ '``', "''", "'s"]
remove += string.punctuation
remove += stopwords.words('english')
remove = set(remove)


def make_wordbag(text):
    return [word.lower() for sent in nltk.tokenize.sent_tokenize(text) for word in nltk.tokenize.word_tokenize(sent) if word.lower() not in remove]



class WordFreq(object):
    """
    Tools for computing and plotting frequencies of word appearances using the nltk.FreqDist method
    """

    def __init__(self, word_list):
        """
        Args:
            word_list: list of words
        """
        self.data = word_list
        self.unique = sorted(set(self.data))

        assert(type(word_list) is list)
        self.freqDist = nltk.FreqDist(self.data)
        self.ranking = list(self.freqDist.keys())

    def topN(self, number_to_display):
        """
        Returns the N most common items in the dataset

        Args:
            number_to_display: The number to display
        """
        assert(type(number_to_display) is int)
        return self.ranking[0: number_to_display]

    def plot(self, number_to_display, **kwargs):
        """
        Displays a plot of the frequency distribution of item frequencies
        Args:
            number_to_display: The top n to display
        """
        assert(type(number_to_display) is int)
        self.freqDist.plot(number_to_display, cumulative=True, **kwargs)

    def compute_individual_word_freq(self):
        """
        Computes the frequency of individual words

        Returns:
            List of dictionaries with the keys 'word' and 'count'
        """
        fd = nltk.FreqDist()
        for word in self.data:
            fd.inc(word)
        results = []
        for w in fd.items():
            results.append({'word': w[0], 'count' : w[1]})
        return results


if __name__ == '__main__':
    pass


"""
Created by adam on 9/20/18
"""
__author__ = 'adam'

import requests
import json
from assets.UrlTools import make_url

import environment


def make_request_header():
    """Creates the request header with Authorization value expected by canvas"""
    return {'Authorization': 'Bearer {}'.format(environment.TOKEN)}


def send_get_request(url, data={}):
    """Makes a get request to the given url, using the data.
    Returns a dictionary from the json
    """
    head = {'Authorization': 'Bearer {}'.format(environment.TOKEN)}
    response = requests.get(url, headers=make_request_header(), json=data)
    return response.json()


def send_post_request(url, data):
    """Makes a post request to the given url, using the data.
    Returns a dictionary from the json
    """
    head = {'Authorization': 'Bearer {}'.format(environment.TOKEN)}
    response = requests.post(url, headers=head, json=data)
    return response.json()


def send_put_request(url, data):
    """Makes a put request to the given url, using the data.
    Returns a dictionary from the json
    """
    response = requests.put(url, headers=make_request_header(), json=data)
    return response.json()


# Assignments
# Get assignment list
def get_all_course_assignments(course_id):
    """Returns a list of all the assignments for the course
    Uses api: GET /api/v1/courses/:course_id/assignments
    """
    url = make_url(course_id, 'assignments')
    url = "%s?per_page=50" % url
    return send_get_request(url, {'include': 'submissions'})


def get_assignment(course_id, assignment_id):
    """Retrieves the specified assignment and returns it as a dictionary"""
    url = make_url(course_id, 'assignments')
    url = "%s/%s" % (url, assignment_id)
    #     print(url)
    return send_get_request(url)


def get_assignments_needing_grading(course_id):
    """Returns a list of tuples (name, id) of assignments which
    have ungraded submissions"""
    assigns = get_all_course_assignments(course_id)
    to_grade = [a for a in assigns if a['needs_grading_count'] >0]

    to_grade = [( g['name'].strip(), g['id']) for g in to_grade]
    return to_grade

if __name__ == '__main__':
    pass